{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "4a53b608-5217-45f4-8a85-06690f68a1dd",
   "metadata": {},
   "outputs": [],
   "source": [
    "import copy\n",
    "from abc import ABC, abstractmethod"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "59c57b4b-7a2f-45a9-a000-29ce809ba3c9",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Building(ABC):\n",
    "    \"\"\"Интерфейс 'Здание'.\n",
    "    От всех наследников требует определения метода '__getitem__' и аттрибута 'rooms'\n",
    "    \"\"\"\n",
    "    @abstractmethod \n",
    "    def __getitem__(self):\n",
    "        \"\"\"Доступ к информации о людях в здании\"\"\"\n",
    "    \n",
    "    @property\n",
    "    @abstractmethod\n",
    "    def rooms(self):\n",
    "        \"\"\"Комнаты здания\"\"\""
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "69d9f20e-6418-4b3d-8322-f6fd45ea3212",
   "metadata": {},
   "outputs": [],
   "source": [
    "class House(Building):\n",
    "    \"\"\"Определение жилого дома\n",
    "    Args:\n",
    "        rooms - [{key(номер квартиры):\"value\"(имя собственника)}, {key:\"value\"},...]\n",
    "    \"\"\"\n",
    "    rooms = []\n",
    "    \n",
    "    def __init__(self, rooms: list):\n",
    "        \"\"\"Задаём начальные параметры жилого дома\"\"\"\n",
    "        self.rooms = rooms\n",
    "    \n",
    "    def __getitem__(self, index: int):\n",
    "        \"\"\"Извлечение информации о жителе по номеру квартиры, в которой он живёт.\n",
    "        Args:\n",
    "            index - Информация о собственнике квартиры под номером index будет извлечена из всех.\n",
    "            copy_obj - копия объекта self. \n",
    "            Копия является новым объектом (хранится на другом участке памяти), \n",
    "            что позволяет взаимодействовать с данными не изменяя их в источнике.\n",
    "            Это возможно с помощью import copy и copy.copy(self)\n",
    "        \"\"\"\n",
    "        copy_obj = copy.copy(self)\n",
    "        copy_obj.rooms = copy_obj.rooms[index]  # Извлечение информации о жителе из квартиры номер index\n",
    "        return copy_obj\n",
    "\n",
    "    def __len__(self):\n",
    "        \"\"\"Возвращает количество квартир в доме\"\"\"\n",
    "        print('В доме квартир:')\n",
    "        return len(self.rooms)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "990a9de8-54c6-417b-a583-858a53411908",
   "metadata": {},
   "outputs": [],
   "source": [
    "house1 = House([{1:\"Максим\"}, {2:\"Никита, Саша\"}, {3:\"Елиза\"}])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 126,
   "id": "3e547cc9-938e-4548-9347-e139191a1f20",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "В доме квартир:\n"
     ]
    },
    {
     "data": {
      "text/plain": [
       "3"
      ]
     },
     "execution_count": 126,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "len(house1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 127,
   "id": "4ba0df41-a759-491b-b9e2-89d95c32201f",
   "metadata": {},
   "outputs": [],
   "source": [
    "class School(Building):\n",
    "    \"\"\"Определение школы\n",
    "    Args:\n",
    "        rooms - {\"key\"(отдел):\"value\"(имя работника отдела), \"key\":\"value\",...}\n",
    "        classes - [{\"key\"(имя учителя):(\"value\"(имя ученика), \"value\", ...)}, {\"key\":(\"value\", \"value\", ...)}, ...]\n",
    "        name - \"Название школы\"\n",
    "    \"\"\"\n",
    "    rooms = {}\n",
    "    classes = []\n",
    "    \n",
    "    def __init__(self, name: str):\n",
    "        \"\"\"Задаём начальные параметры школы\"\"\"\n",
    "        self.name = name\n",
    "        \n",
    "    def __getitem__(self, classroom: str):\n",
    "        \"\"\"Извлечение информации о классе по названию кабинета, в котором он находится.\n",
    "        Args:\n",
    "            classroom: str - извлечение информации о классе находящимся в кабинете. Строка с названием кабинета\n",
    "            copy_obj - копия объекта self. \n",
    "        \"\"\"\n",
    "        copy_obj = copy.copy(self)\n",
    "        copy_obj.rooms = copy_obj.rooms[classroom]  # Извлечение информации о жителе из квартиры номер index\n",
    "        return copy_obj"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 128,
   "id": "fc3d0039-f509-46a5-b062-c3577c2a7a64",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Школа 77\n"
     ]
    }
   ],
   "source": [
    "# Создание новой школы\n",
    "new_school = School(\"Школа 77\")\n",
    "print(new_school.name)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 129,
   "id": "16e95cf4-991f-42eb-9cd3-e976fd8260c5",
   "metadata": {},
   "outputs": [],
   "source": [
    "# Создание классов в виде словарей где ключ-учитель, значение-кортеж с учениками в виде строк\n",
    "class_5a = {\"В. Анатольевич\":(\"Ученик 1\", \"Ученик 2\", \"Ученик 3\")}\n",
    "class_5b = {\"М. Ивановна\":(\"Ученик 4\", \"Ученик 5\", \"Ученик 6\")}"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 130,
   "id": "a9e253f2-a76f-4582-b1bb-81dd672b3119",
   "metadata": {},
   "outputs": [],
   "source": [
    "# Добавление классов с учащимися в школу\n",
    "new_school.classes.append(class_5a)\n",
    "new_school.classes.append(class_5b)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 131,
   "id": "3e401e7c-9359-48ad-8bac-514cc394b1f5",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "[{'В. Анатольевич': ('Ученик 1', 'Ученик 2', 'Ученик 3')}, {'М. Ивановна': ('Ученик 4', 'Ученик 5', 'Ученик 6')}]\n"
     ]
    }
   ],
   "source": [
    "print(new_school.classes)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 132,
   "id": "ec8a112a-bc92-4e92-9419-a4fd725a2b04",
   "metadata": {},
   "outputs": [],
   "source": [
    "# Распределение классов с учащимися в учебные кабинеты\n",
    "new_school.rooms[\"Кабинет математики\"] = class_5a\n",
    "new_school.rooms[\"Кабинет рисования\"] = class_5b"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 133,
   "id": "c5699d58-7173-4b7d-904d-8d7f82af8f7c",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "{'В. Анатольевич': ('Ученик 1', 'Ученик 2', 'Ученик 3')}\n",
      "{'М. Ивановна': ('Ученик 4', 'Ученик 5', 'Ученик 6')}\n"
     ]
    }
   ],
   "source": [
    "print(new_school[\"Кабинет математики\"].rooms)\n",
    "print(new_school[\"Кабинет рисования\"].rooms)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 134,
   "id": "ad72fb3f-4c2c-41eb-9687-e34280442fd0",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Market(Building):\n",
    "    \"\"\"Определение Магазина\n",
    "    Args:\n",
    "        rooms - {\"key\"(отдел):\"value\"(имя работника отдела), \"key\":\"value\",...}\n",
    "        name - \"Название магазина\"\n",
    "    \"\"\"\n",
    "    rooms = []\n",
    "    \n",
    "    def __init__(self, name: str, rooms: list):\n",
    "        \"\"\"Задаём начальные параметры магазина\"\"\"\n",
    "        self.name = name\n",
    "        self.rooms = rooms\n",
    "        \n",
    "    def __getitem__(self, workplace: str):\n",
    "        \"\"\"Извлечение информации о работнике по его месту работы.\n",
    "        Args:\n",
    "            workplace: str - извлечение информации о работнике по его месту работы.\n",
    "            copy_obj - копия объекта self. \n",
    "        \"\"\"\n",
    "        copy_obj = copy.copy(self)\n",
    "        copy_obj.rooms = copy_obj.rooms[workplace]  # Извлечение информации о жителе из квартиры номер index\n",
    "        return copy_obj"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 135,
   "id": "4e16b89f-c49b-4fbf-b348-1111bc8f7578",
   "metadata": {},
   "outputs": [],
   "source": [
    "market = Market(\"Витебские продукты\", {\"Вход\":\"Анатолий\", \"Торговые ряды\":\"Лера\", \"Касса\":\"Аня\"})"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 136,
   "id": "7f9efbb6-6a51-449d-b161-6a777b82e08b",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "{'Вход': 'Анатолий', 'Торговые ряды': 'Лера', 'Касса': 'Аня', 'Склад': 'Морти'}\n",
      "{'Вход': 'Анатолий', 'Торговые ряды': 'Лера', 'Касса': 'Рик', 'Склад': 'Морти'}\n",
      "{'Вход': 'Анатолий', 'Торговые ряды': 'Лера', 'Склад': 'Морти'}\n",
      "{'Вход': 'Анатолий', 'Торговые ряды': 'Лера'}\n"
     ]
    }
   ],
   "source": [
    "# По месту работы в магазине находим ответственного за участок работника\n",
    "workers = market.rooms\n",
    "\n",
    "# Добавить отдел и работника\n",
    "workers[\"Склад\"] = \"Морти\"\n",
    "print(workers)\n",
    "\n",
    "# Добавить/Поменять работника в отделе\n",
    "workers[\"Касса\"] = \"Рик\"\n",
    "print(workers)\n",
    "\n",
    "# Удалить отдела и работника\n",
    "workers.pop(\"Касса\")\n",
    "print(workers)\n",
    "\n",
    "# Удалить последний отдел\n",
    "workers.popitem()\n",
    "print(workers)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 153,
   "id": "b6b82315-c9b6-4766-b20f-51a6db04cb49",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Supply(ABC):\n",
    "    \"\"\"Абстрактный класс 'Снабжение'.\n",
    "    От всех наследников требует определения метода 'delivery' и аттрибута 'delivery_type'\n",
    "    \"\"\"\n",
    "    company = \"Быстро принёс\"\n",
    "        \n",
    "    @abstractmethod \n",
    "    def delivery(self):\n",
    "        \"\"\"Процесс доставки\"\"\""
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 154,
   "id": "67bea975-1f76-47db-b7a5-668ef5699c64",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Ship(Supply):\n",
    "    \"\"\"Доставка товаров кораблём\"\"\"  \n",
    "    def delivery(obj):\n",
    "        print(f'Корабль из компании \"{Ship.company}\" доставил товары в \"{obj.name}\"')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 155,
   "id": "eb1ddc75-39aa-4591-9cbf-e34b96173f60",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Корабль из компании \"Быстро принёс\" доставил товары в \"Витебские продукты\"\n"
     ]
    }
   ],
   "source": [
    "# Доставка в место назначения (в созданный ранее магазин)\n",
    "Ship.delivery(market)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 156,
   "id": "196c0b84-8e64-4a17-9713-3f57cc188af6",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Train(Supply):\n",
    "    \"\"\"Доставка товаров поездом\"\"\"\n",
    "    def delivery(obj):\n",
    "        print(f'Поезд из компании \"{Train.company}\" доставил товары в \"{obj.name}\"')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 157,
   "id": "0a47e82a-1860-4ddb-8f4b-8d6fe255c2f7",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Поезд из компании \"Быстро принёс\" доставил товары в \"Витебские продукты\"\n"
     ]
    }
   ],
   "source": [
    "Train.delivery(market)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 158,
   "id": "52f0f6b4-3cec-4b01-bfea-9798913b9393",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Car(Supply):\n",
    "    \"\"\"Доставка товаров автомобилем\"\"\"\n",
    "    def delivery(obj):\n",
    "        print(f'Машина из компании \"{Car.company}\" доставила товары в \"{obj.name}\"')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 159,
   "id": "7fd09a49-d928-4859-9c08-ce77b883a0f8",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Машина из компании \"Быстро принёс\" доставила товары в \"Витебские продукты\"\n"
     ]
    }
   ],
   "source": [
    "Car.delivery(market)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 160,
   "id": "23f6b893-bcd9-47e0-a509-7514e1ca693b",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Поезд из компании \"Быстро принёс\" доставил товары в \"Витебские продукты\"\n",
      "Машина из компании \"Быстро принёс\" доставила товары в \"Витебские продукты\"\n"
     ]
    }
   ],
   "source": [
    "# Доставка железнодорожным, а потом автомобильным транспортом\n",
    "Train.delivery(market)\n",
    "Car.delivery(market)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "e9b2c8ab-2cf4-4758-8db1-3a5c1fa148e8",
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.6"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
